@@ -18,12 +18,15 @@ NULL
 #'
 #' @return \code{object} with Azimuth results added
 #'
+#' @export
+#' @concept utilities
+#'
 #' @examples
 #' \dontrun{
 #' object <- AddAzimuthResults(object, filename = "azimuth_results.Rds")
 #' }
 #'
-#' @export
+
 AddAzimuthResults <- function(object = NULL, filename) {
   if (is.null(x = filename)) {
     stop("No Azimuth results provided.")
@@ -83,6 +86,8 @@ AddAzimuthResults <- function(object = NULL, filename) {
 #'
 #' @return \code{object} with the mapping scores added
 #'
+#' @keywords internal
+#'
 #' @examples
 #' \dontrun{
 #' object <- AddAzimuthScores(object, filename = "azimuth_pred.tsv")
@@ -132,13 +137,12 @@ AddAzimuthScores <- function(object, filename) {
 #' each module is stored as \code{name#} for each module program present in
 #' \code{features}
 #'
-#' @importFrom ggplot2 cut_number
-#' @importFrom Matrix rowMeans colMeans
-#'
 #' @references Tirosh et al, Science (2016)
 #'
 #' @export
 #' @concept utilities
+#' @rdname AddModuleScore
+#' @method AddModuleScore Seurat
 #'
 #' @examples
 #' \dontrun{
@@ -169,7 +173,7 @@ AddAzimuthScores <- function(object, filename) {
 #' head(x = pbmc_small[])
 #' }
 #'
-AddModuleScore <- function(
+AddModuleScore.Seurat <- function(
   object,
   features,
   pool = NULL,
@@ -189,13 +193,106 @@ AddModuleScore <- function(
   assay.old <- DefaultAssay(object = object)
   assay <- assay %||% assay.old
   DefaultAssay(object = object) <- assay
-  assay.data <- GetAssayData(object = object, assay = assay, slot = slot)
+  features.scores.use <- AddModuleScore(object = object[[assay]],
+                                        features = features,
+                                        kmeans.obj = object@kmeans.obj,
+                                        pool = pool,
+                                        nbin = nbin,
+                                        ctrl = ctrl,
+                                        k = k,
+                                        name = name,
+                                        seed = seed,
+                                        search = search,
+                                        slot = slot,
+                                        ...)
+  object[[colnames(x = features.scores.use)]] <- features.scores.use
+  CheckGC()
+  DefaultAssay(object = object) <- assay.old
+  return(object)
+}
+
+#' @export
+#' @concept utilities
+#' @rdname AddModuleScore
+#' @method AddModuleScore StdAssay
+#' 
+#' @references Tirosh et al, Science (2016)
+#'
+AddModuleScore.StdAssay <- function(
+    object,
+    features,
+    kmeans.obj,
+    pool = NULL,
+    nbin = 24,
+    ctrl = 100,
+    k = FALSE,
+    name = 'Cluster',
+    seed = 1,
+    search = FALSE,
+    slot = 'data',
+    ...
+) {
+  layer_names <- Layers(object, search = slot)
+  input_list <- lapply(
+    layer_names,
+    function(layer_name) {
+      layer_data <- LayerData(object, layer = layer_name)
+      layer_object <- CreateAssayObject(layer_data)
+      return (layer_object)
+    }
+  )
+  output_list <- lapply(
+    input_list,
+    function(input) {
+      AddModuleScore(object = input,
+                     features = features,
+                     kmeans.obj = kmeans.obj,
+                     pool = pool,
+                     nbin = nbin,
+                     ctrl = ctrl,
+                     k = k,
+                     name = name,
+                     seed = seed,
+                     search = search,
+                     slot = slot,
+                     ...)
+    }
+  )
+  features.scores.use <- do.call(rbind,output_list)
+  return(features.scores.use)
+}
+
+#' 
+#' @param kmeans.obj A \code{DoKMeans} output used to define feature clusters 
+#' when \code{k = TRUE}; ignored if \code{k = FALSE}.
+#' @export
+#' @concept utilities
+#' @rdname AddModuleScore
+#' @method AddModuleScore Assay
+#'
+#' @importFrom ggplot2 cut_number
+#' 
+AddModuleScore.Assay <- function(
+    object,
+    features,
+    kmeans.obj,
+    pool = NULL,
+    nbin = 24,
+    ctrl = 100,
+    k = FALSE,
+    name = 'Cluster',
+    seed = 1,
+    search = FALSE,
+    slot = 'data',
+    ...
+) {
+  assay.data <- GetAssayData(object = object, slot = slot)
   features.old <- features
   if (k) {
     .NotYetUsed(arg = 'k')
     features <- list()
-    for (i in as.numeric(x = names(x = table(object@kmeans.obj[[1]]$cluster)))) {
-      features[[i]] <- names(x = which(x = object@kmeans.obj[[1]]$cluster == i))
+    for (i in as.numeric(x = names(x = table(kmeans.obj[[1]]$cluster)))) {
+      features[[i]] <- names(x = which(x = kmeans.obj[[1]]$cluster == i))
     }
     cluster.length <- length(x = features)
   } else {
@@ -315,10 +412,7 @@ AddModuleScore <- function(
   rownames(x = features.scores.use) <- paste0(name, 1:cluster.length)
   features.scores.use <- as.data.frame(x = t(x = features.scores.use))
   rownames(x = features.scores.use) <- colnames(x = object)
-  object[[colnames(x = features.scores.use)]] <- features.scores.use
-  CheckGC()
-  DefaultAssay(object = object) <- assay.old
-  return(object)
+  return(features.scores.use)
 }
 
 #' Aggregated feature expression by identity class
@@ -981,7 +1075,7 @@ GroupCorrelation <- function(
   grp.cors <- as.data.frame(x = grp.cors[which(x = !is.na(x = grp.cors))])
   grp.cors$gene_grp <- gene.grp[rownames(x = grp.cors)]
   colnames(x = grp.cors) <- c(paste0(var, "_cor"), "feature.grp")
-  object[[assay]][] <- grp.cors
+  object[[assay]] <- AddMetaData(object[[assay]], grp.cors)
   if (isTRUE(x = do.plot)) {
     print(GroupCorrelationPlot(
       object = object,
@@ -1166,11 +1260,31 @@ PercentageFeatureSet <- function(
   assay = NULL
 ) {
   assay <- assay %||% DefaultAssay(object = object)
-  if (!is.null(x = features) && !is.null(x = pattern)) {
-    warn(message = "Both pattern and features provided. Pattern is being ignored.")
+  if (!is.null(x = features))  {
+    if (!is.null(x = pattern)) {
+      warning(
+        paste(
+          "Both `features` and `pattern` were provided;",
+          "`pattern` will be ignored."
+        )
+      )
+    }
+
+    available_features <- Features(object[[assay]])
+    missing_features <- setdiff(features, available_features)
+    if (length(missing_features) > 0) {
+      warning(
+        paste(
+          "The following features are not found in the",
+          paste0("'", assay, "'"),
+          "assay:",
+          paste(paste0("'", missing_features, "'"), collapse = ", ")
+        )
+      )
+    }
   }
   percent.featureset <- list()
-  layers <- Layers(object = object, search = "counts")
+  layers <- Layers(object = object, assay = assay, search = "counts")
   for (i in seq_along(along.with = layers)) {
     layer <- layers[i]
     features.layer <- features %||% grep(
@@ -1180,6 +1294,7 @@ PercentageFeatureSet <- function(
     layer.data <- LayerData(object = object,
                             assay = assay,
                             layer = layer)
+    features.layer <- intersect(features.layer, rownames(layer.data))
     layer.sums <- colSums(x = layer.data[features.layer, , drop = FALSE])
     layer.perc <- layer.sums / object[[]][colnames(layer.data), paste0("nCount_", assay)] * 100
     percent.featureset[[i]] <- layer.perc
@@ -1192,30 +1307,29 @@ PercentageFeatureSet <- function(
   return(percent.featureset)
 }
 
-# Pseudobulk feature expression by identity class
+#' Pseudobulk feature expression by identity class
+#'
+#' Returns a representative expression value for each identity class
+#'
+#' @param object Seurat object
+#' @param method Whether to 'average' (default) or 'aggregate' expression levels
+#' @param assay  The name of the passed assay - used primarily for warning/error messages
+#' @param category.matrix A matrix defining groupings for pseudobulk expression
+#' calculations; each column represents an identity class, and each row a sample
+#' @param features Features to analyze. Default is all features in the assay
+#' @param layer Layer(s) to user; if multiple are given, assumed to follow
+#' the order of 'assays' (if specified) or object's assays
+#' @param slot (Deprecated) See \code{layer}
+#' @param verbose Print messages and show progress bar
+#' @param ... Arguments to be passed to methods such as \code{\link{CreateSeuratObject}}
 #
-# Returns a representative expression value for each identity class
-#
-# @param object Seurat object
-# @param method Whether to 'average' (default) or 'aggregate' expression levels
-# @param assays Which assays to use. Default is all assays
-# @param features Features to analyze. Default is all features in the assay
-# @param return.seurat Whether to return the data as a Seurat object. Default is FALSE
-# @param group.by Categories for grouping (e.g, ident, replicate, celltype); 'ident' by default
-# @param add.ident (Deprecated) Place an additional label on each cell prior to pseudobulking
-# (very useful if you want to observe cluster pseudobulk values, separated by replicate, for example)
-# @param slot Slot(s) to use; if multiple slots are given, assumed to follow
-# the order of 'assays' (if specified) or object's assays
-# @param verbose Print messages and show progress bar
-# @param ... Arguments to be passed to methods such as \code{\link{CreateSeuratObject}}
-#
-# @return Returns a matrix with genes as rows, identity classes as columns.
-# If return.seurat is TRUE, returns an object of class \code{\link{Seurat}}.
+#' @return Returns a matrix with genes as rows, identity classes as columns.
+#' If return.seurat is TRUE, returns an object of class \code{\link{Seurat}}.
 #' @method PseudobulkExpression Assay
-#' @importFrom SeuratObject .IsFutureSeurat
+#' @rdname PseudobulkExpression
 #' @export
-#
-#
+#' @concept utilities
+#'
 PseudobulkExpression.Assay <- function(
   object,
   assay,
@@ -1227,14 +1341,7 @@ PseudobulkExpression.Assay <- function(
   ...
 ) {
   if (is_present(arg = slot)) {
-    f <- if (.IsFutureSeurat(version = '5.1.0')) {
-      deprecate_stop
-    } else if (.IsFutureSeurat(version = '5.0.0')) {
-      deprecate_warn
-    } else {
-      deprecate_soft
-    }
-    f(
+    deprecate_soft(
       when = '5.0.0',
       what = 'GetAssayData(slot = )',
       with = 'GetAssayData(layer = )'
@@ -1278,9 +1385,10 @@ PseudobulkExpression.Assay <- function(
 }
 
 #' @method PseudobulkExpression StdAssay
+#' @rdname PseudobulkExpression
 #' @export
-#
-#
+#' @concept utilities
+#'
 PseudobulkExpression.StdAssay <- function(
   object,
   assay,
@@ -1292,14 +1400,7 @@ PseudobulkExpression.StdAssay <- function(
   ...
 ) {
   if (is_present(arg = slot)) {
-    f <- if (.IsFutureSeurat(version = '5.1.0')) {
-      deprecate_stop
-    } else if (.IsFutureSeurat(version = '5.0.0')) {
-      deprecate_warn
-    } else {
-      deprecate_soft
-    }
-    f(
+    deprecate_soft(
       when = '5.0.0',
       what = 'GetAssayData(slot = )',
       with = 'GetAssayData(layer = )'
@@ -1357,10 +1458,22 @@ PseudobulkExpression.StdAssay <- function(
   return(data.return)
 }
 
-
+#' @param assays Which assays to use. Default is all assays
+#' @param return.seurat Whether to return the data as a Seurat object. Default is FALSE
+#' @param group.by Categories for grouping (e.g, "ident", "replicate",
+#' "celltype"); "ident" by default
+#' @param add.ident (Deprecated) See group.by
+#' @param method The method used for calculating pseudobulk expression; one of:
+#' "average" or "aggregate"
+#' @param normalization.method Method for normalization, see \code{\link{NormalizeData}}
+#' @param scale.factor Scale factor for normalization, see \code{\link{NormalizeData}}
+#' @param margin Margin to perform CLR normalization, see \code{\link{NormalizeData}}
+#'
 #' @method PseudobulkExpression Seurat
-#' @importFrom SeuratObject .IsFutureSeurat
+#' @rdname PseudobulkExpression
 #' @export
+#' @concept utilities
+#'
 PseudobulkExpression.Seurat <- function(
   object,
   assays = NULL,
@@ -1386,14 +1499,7 @@ PseudobulkExpression.Seurat <- function(
     stop("'method' must be either 'average' or 'aggregate'")
   }
   if (is_present(arg = slot)) {
-    f <- if (.IsFutureSeurat(version = '5.1.0')) {
-      deprecate_stop
-    } else if (.IsFutureSeurat(version = '5.0.0')) {
-      deprecate_warn
-    } else {
-      deprecate_soft
-    }
-    f(
+    deprecate_soft(
       when = '5.0.0',
       what = 'AverageExpression(slot = )',
       with = 'AverageExpression(layer = )'
@@ -1458,26 +1564,28 @@ PseudobulkExpression.Seurat <- function(
     group.by <- colnames(x = data)[which(num.levels > 1)]
     data <- data[, which(num.levels > 1), drop = F]
   }
-  category.matrix <- CreateCategoryMatrix(labels = data, method = method)
-  #check if column names are numeric
-  col.names <- colnames(category.matrix)
-  if (any(!(grepl("^[a-zA-Z]|^\\.[^0-9]", col.names)))) {
-    col.names <- ifelse(
-      !(grepl("^[a-zA-Z]|^\\.[^0-9]", col.names)),
-      paste0("g", col.names),
-      col.names
-    )
-    colnames(category.matrix) <- col.names
-    inform(
-      message = paste0("First group.by variable `", group.by[1],
-      "` starts with a number, appending `g` to ensure valid variable names"),
-      .frequency = "regularly",
-      .frequency_id = "PseudobulkExpression"
-    )
-  }
-
   data.return <- list()
   for (i in 1:length(x = assays)) {
+    data_sub <- data[intersect(rownames(data),Cells(object[[assays[i]]])),,drop=FALSE]
+    category.matrix <- CreateCategoryMatrix(labels = data_sub, method = method)
+    #check if column names are numeric
+    col.names <- colnames(category.matrix)
+    if (any(!(grepl("^[a-zA-Z]|^\\.[^0-9]", col.names)))) {
+      col.names <- ifelse(
+        !(grepl("^[a-zA-Z]|^\\.[^0-9]", col.names)),
+        paste0("g", col.names),
+        col.names
+      )
+      colnames(category.matrix) <- col.names
+      inform(
+        message = paste0("First group.by variable `", group.by[1],
+        "` starts with a number, appending `g` to ensure valid variable names"),
+        .frequency = "regularly",
+        .frequency_id = "PseudobulkExpression"
+      )
+    }
+
+
     data.return[[assays[i]]] <- PseudobulkExpression(
       object = object[[assays[i]]],
       assay = assays[i],
@@ -1528,6 +1636,8 @@ PseudobulkExpression.Seurat <- function(
         ) <- NormalizeData(
           as.matrix(x = data.return[[1]]),
           normalization.method = normalization.method,
+          scale.factor = scale.factor,
+          margin = margin,
           verbose = verbose
         )
       }
@@ -2827,6 +2937,7 @@ SweepNonzero <- function(
 #' @importFrom Matrix colSums sparse.model.matrix
 #' @importFrom stats as.formula
 #' @export
+#' @concept utilities
 #'
 CreateCategoryMatrix <- function(
   labels,
@@ -2913,6 +3024,7 @@ CreateCategoryMatrix <- function(
 #' @param fov FOV object to gather cell positions from
 #' @param group.by Cell classifications to count in spatial neighborhood
 #' @param assay Name for spatial neighborhoods assay
+#' @param cluster.name Name of output clusters
 #' @param neighbors.k Number of neighbors to consider for each cell
 #' @param niches.k Number of clusters to return based on the niche assay
 #'
@@ -2926,45 +3038,46 @@ BuildNicheAssay <- function(
   fov,
   group.by,
   assay = "niche",
+  cluster.name = "niches",
   neighbors.k = 20,
   niches.k = 4
 ) {
-  # find neighbors based on tissue position
-  coords <- GetTissueCoordinates(object[[fov]], which = "centroids")
-  cells <- coords$cell
-  rownames(coords) <- cells
-  coords <- as.matrix(coords[ , c("x", "y")])
-  neighbors <- FindNeighbors(coords, k.param = neighbors.k)
-  neighbors$nn <- neighbors$nn[Cells(object), Cells(object)]
-
-  # build cell x cell type matrix
-  ct.mtx <- matrix(
+  # initialize an empty cells x groups binary matrix
+  cells <- Cells(object[[fov]])
+  group.labels <- unlist(object[[group.by]][cells, ])
+  groups <- sort(unique(group.labels))
+  cell.type.mtx <- matrix(
     data = 0,
     nrow = length(cells),
-    ncol = length(unlist(unique(object[[group.by]])))
+    ncol = length(groups)
   )
-  rownames(ct.mtx) <- cells
-  colnames(ct.mtx) <- unique(unlist(object[[group.by]]))
-  cts <- object[[group.by]]
-  for (i in 1:length(cells)) {
-    ct <- as.character(cts[cells[[i]], ])
-    ct.mtx[cells[[i]], ct] <- 1
-  }
+  rownames(cell.type.mtx) <- cells
+  colnames(cell.type.mtx) <- groups
+  # populate the binary matrix
+  cells.idx <- seq_along(cells)
+  group.idx <- match(group.labels, groups)
+  cell.type.mtx[cbind(cells.idx, group.idx)] <- 1
+
+  # find neighbors based on tissue position
+  coords <- GetTissueCoordinates(object[[fov]], which = "centroids")
+  rownames(coords) <- coords[["cell"]]
+  coords <- as.matrix(coords[ , c("x", "y")])
+  neighbors <- FindNeighbors(coords, k.param = neighbors.k, compute.SNN = FALSE)
 
   # create niche assay
-  sum.mtx <- as.matrix(neighbors$nn %*% ct.mtx)
+  sum.mtx <- as.matrix(neighbors[["nn"]] %*% cell.type.mtx)
   niche.assay <- CreateAssayObject(counts = t(sum.mtx))
   object[[assay]] <- niche.assay
   DefaultAssay(object) <- assay
 
-  # cluster niches assay
+  # cluster niche assay
   object <- ScaleData(object)
   results <- kmeans(
     x = t(object[[assay]]@scale.data),
     centers = niches.k,
     nstart = 30
   )
-  object$niches <- results[["cluster"]]
+  object[[cluster.name]] <- results[["cluster"]]
 
   return(object)
 }
