@@ -1,3 +1,26 @@
+#' @importFrom progressr progressor
+#' @importFrom methods slot slot<-
+#' @importFrom lifecycle deprecated deprecate_soft deprecate_stop
+#' deprecate_warn is_present
+#' @importFrom rlang !!!
+#' abort
+#' arg_match
+#' arg_match0
+#' as_name
+#' caller_env
+#' check_installed
+#' enquo
+#' inform
+#' is_integerish
+#' is_na
+#' is_quosure
+#' is_scalar_integerish
+#' quo_get_env
+#' quo_get_expr
+#' warn
+#'
+NULL
+
 #' @section Package options:
 #'
 #' Seurat uses the following [options()] to configure behaviour:
@@ -24,19 +47,31 @@
 #' @docType package
 #' @rdname Seurat-package
 #' @name Seurat-package
-#' @keywords internal
 #'
 "_PACKAGE"
 
+#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+# Options
+#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 seurat_default_options <- list(
   Seurat.memsafe = FALSE,
   Seurat.warn.umap.uwot = TRUE,
   Seurat.checkdots = "warn",
-  Seurat.limma.wilcox.msg = TRUE,
+  Seurat.presto.wilcox.msg = TRUE, #CHANGE
   Seurat.Rfast2.msg = TRUE,
-  Seurat.warn.vlnplot.split = TRUE
+  Seurat.warn.vlnplot.split = TRUE,
+  Seurat.object.assay.version = "v5"
 )
 
+
+#' @importFrom methods setClassUnion
+#' @importClassesFrom Matrix dgCMatrix
+#'
+NULL
+
+setClassUnion(name = 'V3Matrix', members = c('matrix', 'dgCMatrix'))
+
 AttachDeps <- function(deps) {
   for (d in deps) {
     if (!paste0('package:', d) %in% search()) {
@@ -46,13 +81,25 @@ AttachDeps <- function(deps) {
   }
 }
 
+#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+# Hooks
+#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+#' @importFrom SeuratObject AttachDeps
+#'
 .onAttach <- function(libname, pkgname) {
-  AttachDeps(deps = 'SeuratObject')
+  AttachDeps(deps = c('SeuratObject'))
+  return(invisible(x = NULL))
 }
 
 .onLoad <- function(libname, pkgname) {
-  op <- options()
-  toset <- !(names(x = seurat_default_options) %in% names(x = op))
-  if (any(toset)) options(seurat_default_options[toset])
-  invisible(x = NULL)
+  toset <- setdiff(
+    x = names(x = seurat_default_options),
+    y = names(x = options())
+  )
+  # toset <- names(x = seurat_default_options)
+  if (length(x = toset)) {
+    options(seurat_default_options[toset])
+  }
+  return(invisible(x = NULL))
 }
