@@ -397,28 +397,6 @@ test_that("SCTransform v1 works as expected", {
   expect_equal(fa["MS4A1", "residual_variance"], 2.875761, tolerance = 1e-6)
 })
 
-test_that("SCTransform v2 works as expected", {
-  skip_on_cran()
-  skip_if_not_installed("glmGamPoi")
-
-  object <- suppressWarnings(SCTransform(object = object, verbose = FALSE, vst.flavor = "v2",  seed.use = 1448145))
-
-  expect_true("SCT" %in% names(object))
-  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], layer = "scale.data"))[1]), 24.5183, tolerance = 1e-2)
-  expect_equal(as.numeric(rowSums(GetAssayData(object = object[["SCT"]], layer = "scale.data"))[5]), 0)
-  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], layer = "data"))[1]), 58.65829, tolerance = 1e-6)
-  expect_equal(as.numeric(rowSums(GetAssayData(object = object[["SCT"]], layer = "data"))[5]), 13.75449, tolerance = 1e-6)
-  expect_equal(as.numeric(colSums(GetAssayData(object = object[["SCT"]], layer = "counts"))[1]), 141)
-  expect_equal(as.numeric(rowSums(GetAssayData(object = object[["SCT"]], layer = "counts"))[5]), 40)
-  expect_equal(length(VariableFeatures(object[["SCT"]])), 220)
-  fa <- SCTResults(object = object, assay = "SCT", slot = "feature.attributes")
-  expect_equal(fa["MS4A1", "detection_rate"], 0.15)
-  expect_equal(fa["MS4A1", "gmean"], 0.2027364, tolerance = 1e-6)
-  expect_equal(fa["MS4A1", "variance"], 1.025158, tolerance = 1e-6)
-  expect_equal(fa["MS4A1", "residual_mean"], 0.2763993, tolerance = 1e-6)
-  expect_equal(fa["MS4A1", "residual_variance"], 3.023062, tolerance = 1e-6)
-})
-
 suppressWarnings(RNGversion(vstr = "3.5.0"))
 object <- suppressWarnings(SCTransform(object = object, vst.flavor = "v1", ncells = 80, verbose = FALSE, seed.use =  42))
 test_that("SCTransform ncells param works", {
@@ -473,6 +451,41 @@ test_that("SCTransform v2 works as expected", {
   expect_equal(fa["FCER2", "theta"], Inf)
 })
 
+test_that("SCTransform `clip.range` param works as expected", {
+  # make a copy of the testing data
+  test.data <- object
+  # override defaults for ease of testing
+  clip.min <- -0.1
+  clip.max <- 0.1
+
+  # for some reason, the clipping seems to be a little fuzzy at the upper end,
+  # since this is expected behaviour we'll need to accomodate the difference
+  clip.max.tolerance <- 0.1
+
+  test.result <- suppressWarnings(
+      SCTransform(
+      test.data,
+      clip.range = c(clip.min, clip.max),
+    )
+  )
+  scale.data <- LayerData(test.result[["SCT"]], layer = "scale.data")
+  expect_true(min(scale.data) >= clip.min)
+  expect_true(max(scale.data) <= (clip.max + clip.max.tolerance))
+
+  # when `ncells` is less than the size of the dataset the residuals will get 
+  # re-clipped in batches, make sure this clipping is done correctly as well
+  test.result <- suppressWarnings(
+    SCTransform(
+      test.data,
+      clip.range = c(clip.min, clip.max),
+      ncells = 40
+    )
+  )
+  scale.data <- LayerData(test.result[["SCT"]], layer = "scale.data")
+  expect_true(min(scale.data) >= clip.min)
+  expect_true(max(scale.data) <= (clip.max + clip.max.tolerance))
+})
+
 test_that("SCTransform `vars.to.regress` param works as expected", {
   # make a copy of the testing data
   test.data <- object
@@ -537,6 +550,32 @@ test_that("SCTransform `vars.to.regress` param works as expected", {
   expect_false(identical(left[["SCT"]]$scale.data, right[["SCT"]]$scale.data))
 })
 
+fake.meta.data2 <- data.frame(rep(1:2, ncol(pbmc.test)/2))
+rownames(fake.meta.data2) <- colnames(pbmc.test)
+colnames(fake.meta.data2) <- "Condition"
+object2 <- CreateSeuratObject(counts = pbmc.test,
+                             meta.data = fake.meta.data2)
+
+test_that("`SCTransform` is consistent for multi-layer inputs", {
+  clip.range = c(-1.632993, 1.632993)
+  
+  test_case_v3 <- SplitObject(object2, split.by = "Condition")
+  result_v3_list <- lapply(test_case_v3, SCTransform, clip.range = clip.range)
+  result_v3 <- merge(result_v3_list[[1]], result_v3_list[[2]])
+  
+  test_case_v5 <- object2
+  test_case_v5 <- split(object2, f = object2$Condition)
+  result_v5 <- SCTransform(test_case_v5, clip.range = clip.range)
+  
+  expected <- result_v3[["SCT"]]@data[,colnames(object2)]
+  result <- result_v5[["SCT"]]@data[,colnames(object2)]
+  expect_true(all.equal(expected, result))
+  
+  expected <- result_v3[["SCT"]]@scale.data[,colnames(object2)]
+  result <- result_v5[["SCT"]]@scale.data[,colnames(object2)]
+  expect_true(all.equal(expected[rownames(result), ], result))
+})
+
 test_that("SCTransform is equivalent for BPcells ", {
   skip_on_cran()
   skip_on_cran()
